import json
import unicodecsv as csv
from datetime import datetime
from .util import flatten
import sys

_format = "jsonl"
_output = sys.stdout.buffer
_csv_writer = None


def set_format(format):
    global _format, _csv_writer

    _format = format
    _csv_writer = None


def set_output(output):
    global _output, _csv_writer

    _output = output
    _csv_writer = None


def disable_standard_output():
    result_listeners.remove(write_standard_output)


def write_standard_output(obj: dict):
    global _csv_writer

    if _format == "jsonl":
        _output.write((json.dumps(obj) + "\n").encode("utf-8"))
    elif _format == "csv":
        obj = flatten(obj)
        if _csv_writer == None:
            _csv_writer = csv.DictWriter(_output, obj.keys())
            _csv_writer.writeheader()
        _csv_writer.writerow(obj)


result_listeners = [write_standard_output]  # Functions to call on each result


def emit(obj: dict):
    global result_listeners

    # Serialize keys
    for key in obj.keys():
        if type(obj[key]) == datetime:
            obj[key] = datetime.isoformat(obj[key])

    for listener in result_listeners:
        listener(obj)


def close():
    _output.close()



#!/usr/bin/env python3

import click
from . import output
from .util import (
    add_click_options,
    COMMENT_PAGE_OPTIONS,
)
from .output import emit
from .capabilities import comments, user, url, trending


@click.group()
@click.option(
    "--format",
    default="jsonl",
    help="output format",
    type=click.Choice(["jsonl", "csv"]),
)
def cli(format):
    output.set_format(format)


@click.command("comments", help="Pull all the most recent comments")
@add_click_options(COMMENT_PAGE_OPTIONS)
def comments_command(**kwargs):
    return comments.fetch(**kwargs)


@click.command("trending", help="Pull the current trending URLs")
def trending_command():
    return trending.fetch()


@click.command(
    "url",
    help="Pull comments for a particular URL. Note that several comment metadata items (such as upvotes, downvotes, and comments) are not available when pulling comments from a URL.",
)
@click.argument("url")
@add_click_options(COMMENT_PAGE_OPTIONS)
def url_command(**kwargs):
    return url.fetch(**kwargs)


@click.command(
    "user", help="Pull all the comments of a particular user, identified by their UID"
)
@click.argument("user")
@add_click_options(COMMENT_PAGE_OPTIONS)
def user_command(**kwargs):
    return user.fetch(**kwargs)


cli.add_command(comments_command)
cli.add_command(trending_command)
cli.add_command(url_command)
cli.add_command(user_command)



import requests
from ..output import emit


def fetch():
    data = requests.get("https://dissenter.com/url").json()

    # We currently ignore trending comments.
    # The pagination here is opaque/doesn't seem to be meaningful, so we only pull once
    for url in data["trendingUrls"]:
        emit(url)



from ..util import (
    parse_date,
    pull_comments_page,
)
from ..output import emit


def fetch(user=None, sort="latest", after_id="0", after_time="Jan 1, 2000", max=None):
    after_id = int(after_id.strip(), 16)
    after_time = parse_date(after_time)

    page = 1
    total_emitted = 0
    while (
        len(
            page_comments := pull_comments_page(
                s=sort, p=page, v="profile", cpp=350, user=user
            )
        )
        > 0
    ):
        for comment in page_comments:
            if int(comment["id"], 16) < after_id:
                return
            if comment["time"] < after_time:
                return

            emit(comment)
            total_emitted += 1

            if max is not None:
                if total_emitted >= max:
                    return

        page += 1



from dateparser import parse as date_parse
from datetime import datetime
import requests
from bs4 import BeautifulSoup
import urllib
import click
import collections

COMMENT_PAGE_OPTIONS = [
    click.option(
        "--sort",
        default="latest",
        help="comment sort order",
        type=click.Choice(["latest", "controversial", "top"]),
    ),
    click.option(
        "--after-id", default="0", help="pull no earlier than this comment ID", type=str
    ),
    click.option(
        "--after-time",
        default="Jan 1, 2000",
        help="pull no comments posted earlier than this time",
        type=str,
    ),
    click.option("--max", help="maximum number of comments to pull", type=int),
]


def flatten(d, parent_key="", sep="_"):
    items = []
    for k, v in d.items():
        new_key = parent_key + sep + k if parent_key else k
        if isinstance(v, collections.MutableMapping):
            items.extend(flatten(v, new_key, sep=sep).items())
        else:
            items.append((new_key, v))
    return dict(items)


def parse_date(text: str) -> datetime:
    if text == "a few seconds ago":
        text = "now"

    return date_parse(text)


def add_click_options(options):
    def _add_options(func):
        for option in reversed(options):
            func = option(func)
        return func

    return _add_options


def parse_int(text: str) -> int:
    text = text.replace(",", "")
    return int(text)


def try_get(func):
    try:
        return func()
    except:
        return None


def pull_url_id(url: str) -> str:
    resp = requests.get(
        f"https://dissenter.com/discussion/begin?"
        + urllib.parse.urlencode({"url": url})
    )
    soup = BeautifulSoup(resp.text, features="html.parser")
    return try_get(
        lambda: soup.find("div", {"data-commenturl-id": True}).get("data-commenturl-id")
    )


def pull_comments_page(**kwargs):
    resp = requests.get(
        f"https://dissenter.com/comment?" + urllib.parse.urlencode(kwargs)
    )
    soup = BeautifulSoup(resp.text, features="html.parser")

    comments = []
    for comment_elem in soup.find_all("div", class_="comment-container"):
        comments.append(
            {
                "id": try_get(lambda: comment_elem.get("data-comment-id").strip()),
                "author_id": try_get(
                    lambda: comment_elem.get("data-author-id").strip()
                ),
                "author_name": try_get(
                    lambda: comment_elem.find("span", class_="profile-name").text
                ),
                "author_url": try_get(
                    lambda: comment_elem.find("a", {"target": "dissenter-profile"}).get(
                        "href"
                    )
                ),
                "url": try_get(
                    lambda: comment_elem.find("a", {"target": "dissenter-item"}).get(
                        "href"
                    )
                ),
                "url_upvotes": try_get(
                    lambda: parse_int(
                        comment_elem.find("span", class_="stat-upvotes").text
                    )
                ),
                "url_downvotes": try_get(
                    lambda: parse_int(
                        comment_elem.find("span", class_="stat-downvotes").text
                    )
                ),
                "url_comments": try_get(
                    lambda: parse_int(
                        comment_elem.find(
                            "div", class_="row no-gutters align-items-center ml-auto"
                        ).text.split(" ")[0]
                    )
                ),
                "text": try_get(
                    lambda: comment_elem.find("div", class_="comment-body").text
                ),
                "replies": try_get(
                    lambda: parse_int(
                        comment_elem.find("span", class_="stat-replies").text
                    )
                ),
                "time": try_get(
                    lambda: parse_date(
                        comment_elem.find("a", {"title": "View comment"})
                        .find("span")
                        .text
                    )
                ),
            }
        )

    return comments



from .output import set_output, set_format, result_listeners, disable_standard_output
from .capabilities.comments import fetch as comments
from .capabilities.trending import fetch as trending
from .capabilities.url import fetch as url
from .capabilities.user import fetch as user



import pathlib
from setuptools import setup, find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="ressenter",
    version="0.0.10",
    description="Toolkit for Dissenter",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/stanfordio/ressenter",
    author_email="mccain@stanford.edu",
    license="Apache License 2.0",
    classifiers=[
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        "beautifulsoup4",
        "requests",
        "click",
        "unicodecsv",
        "dateparser",
    ],
    entry_points={
        "console_scripts": [
            "ressenter=ressenter.cli:cli",
        ]
    },
    python_requires='>=3.8'
)
